function [par_all,par_loc,par_dim,par_iter,par_rank,par_mog,par_w,par_xi,par_model,par_others]=subf_par_init(Y,par)
%% par_others
%
[data_type,data_type_given]=par_extract_mth(par,'data_type','syn');
input_check_mth(data_type,'c',{'syn','msi','hsi','real'})
if data_type_given
    fprintf(['Use given data_type ' data_type '\n']);
else
    warning('No data_type given, use default synthetic data setting')
end
%
X_ori=par_extract_mth(par,'X_ori',zeros(size(Y)));
input_check_mth(X_ori,'n',{size(Y)})
%
detail_show=par_extract_mth(par,'detail_show',false,@(x)logical(x));
input_check_mth(detail_show,'l',{1})
%
par_others=[];
par_others.data_type=data_type;
par_others.X_ori=X_ori;
par_others.detail_show=detail_show;
%% par_loc
%
miss_idx=par_extract_mth(par,'miss_idx',false(size(Y)));
input_check_mth(miss_idx,'l',numel(Y))
%
true_idx=par_extract_mth(par,'true_idx',false(size(Y)));
input_check_mth(true_idx,'l',numel(Y))
if any(miss_idx(:) & true_idx(:))
    error('miss_idx and true_idx are overlapped')
end
%
noise_idx=~(true_idx | miss_idx);
%
par_loc=[];
par_loc.miss=miss_idx;
par_loc.true=true_idx;
par_loc.noise=noise_idx;
%% par_dim
%
method=par_extract_mth(par,'method',{'tucker'});
if ~iscell(method);method={method};end
%
dim_num=0;
unfold_funchdl={};
fold_funchdl={};
row_num=[];
for i=1:numel(method)
    switch method{i}
        case {'tucker','Tucker'}
            dim_num_temp=ndims(Y);
            if dim_num_temp==2
                dim_num_temp=1;
            end
            dim_num=dim_num+dim_num_temp;
            for j=1:dim_num_temp
                unfold_funchdl=[unfold_funchdl {str2func(['@(x)unfold_mth(x,' num2str(j) ')'])}];%@(x)unfold_mth(x,j)
                fold_funchdl=[fold_funchdl {str2func(['@(x)fold_mth(x,' mat2str(size(Y)) ',' num2str(j) ')'])}];%@(x)fold_mth(x,size(Y),j)
                row_num=[row_num size(Y,j)];
            end
        case {'train','Train'}
            Y_vdt=vdt_mth(Y,par.factor);
            Y_vdt_size=size(Y_vdt);
            dim_num_temp=ndims(Y_vdt)-1;
            dim_num=dim_num+dim_num_temp;
            for j=1:dim_num_temp
                unfold_funchdl=[unfold_funchdl {str2func(['@(x)reshape(vdt_mth(x,' mat2str(par.factor) '),' num2str(prod(Y_vdt_size(1:j))) ',[])'])}];
                %@(x)reshape(vdt_mth(x,par.factor),prod(Y_vdt_size(1:j)),[])
                fold_funchdl=[fold_funchdl {str2func(['@(x)vdt_mth(reshape(x,' mat2str(Y_vdt_size) '),' mat2str(par.factor) ',1)'])}];
                %@(x)vdt_mth(reshape(x,Y_vdt_size),par.factor,1)
                row_num=[row_num prod(Y_vdt_size(1:j))];
            end
        otherwise
            error('unknown unfold method')
    end
end
col_num=numel(Y)./row_num;
%
par_dim=[];
par_dim.num=dim_num;
par_dim.unfold=unfold_funchdl;
par_dim.fold=fold_funchdl;
par_dim.row=row_num;
par_dim.col=col_num;
%% par_iter
%
stage_all=par_extract_mth(par,'stage_all',[1 2]);
if ~(isequal(stage_all,1) || isequal(stage_all,2) || isequal(stage_all,[1,2]))
    error('wrong stage_all')
end
%
switch data_type
    case 'syn'
        iter_max_default=200;
    case 'msi'
        iter_max_default=100;
    case {'hsi','real'}
        iter_max_default=200;   
end
iter_max=par_extract_mth(par,'iter_max',iter_max_default);
input_check_mth(iter_max,'n',{1},1,[],1)
%
switch data_type
    case 'syn'
        tol=par_extract_mth(par,'tol',[1e-3,1e-4]);
        tol_X_minus_UVt=par_extract_mth(par,'tol_X_minus_UVt',1e-3);
    case 'msi'
        tol=par_extract_mth(par,'tol',[5e-3,5e-4]);
        tol_X_minus_UVt=par_extract_mth(par,'tol_X_minus_UVt',5e-3);
    case 'hsi'
        tol=par_extract_mth(par,'tol',[5e-3,1e-4]);
        tol_X_minus_UVt=par_extract_mth(par,'tol_X_minus_UVt',5e-3);   
    case 'real'
        tol=par_extract_mth(par,'tol',[1e-3,1e-4]);
        tol_X_minus_UVt=par_extract_mth(par,'tol_X_minus_UVt',1e-3);
end
input_check_mth(tol,'n',{2},0,1)
input_check_mth(tol_X_minus_UVt,'n',{1},0,1)
%
switch data_type
    case 'syn'
        uv_init_stage2_default='noisy';
    case 'msi'
        uv_init_stage2_default='noisy';
    case {'hsi','real'}
        uv_init_stage2_default='stage1';
end
uv_init_stage2=par_extract_mth(par,'uv_init_stage2',uv_init_stage2_default);
input_check_mth(uv_init_stage2,'c',{'noisy','random','stage1','given'})
if strcmp(uv_init_stage2,'stage1') && ~ismember(1,stage_all)
    error('wrong uv_init_stage2')
end
if strcmp(uv_init_stage2,'given')
    if isfield(par,'Y_ini')
        Y_ini=par.Y_ini;
    else
        error('no Y_ini for uv_init_stage2=''given''')
    end
end
%
par_iter=[];
par_iter.stage=stage_all;
par_iter.max=iter_max;
par_iter.tol=tol;
par_iter.tol_X_minus_UVt=tol_X_minus_UVt;
par_iter.uv_init_stage2=uv_init_stage2;
if exist('Y_ini','var');par_iter.Y_ini=Y_ini;end
%% par_rank
%
rank_max=min(row_num,col_num);
%
switch data_type
    case 'syn'
        rank_init1_default=0.8*ones(1,dim_num);
        rank_init2_default=0.5*ones(1,dim_num);
    case 'msi'
        rank_init1_default=0.5*ones(1,dim_num);
        rank_init2_default=0.3*ones(1,dim_num);
    case 'hsi'
        rank_init1_default=0.5*ones(1,dim_num);
        rank_init2_default=0.3*ones(1,dim_num);
    case 'real'
        rank_init1_default=[0.3 0.3 0.3];%
        rank_init2_default=[0.3 0.3 0.3];%
end
rank_init=par_extract_mth(par,'rank_init',min(max(15,ceil(rank_init1_default.*rank_max)),rank_max-1));
input_check_mth(rank_init,'n',{dim_num},1,rank_max,1)
rank_init2=min(max(15,ceil(rank_init2_default.*rank_max)),rank_max-1);
%
if isequal(stage_all,2)
    if isfield(par,'rank_prior')
        rank_prior=par.rank_prior(:)';
    else
        error('no rank_prior for stage_all=''2''')
    end
    input_check_mth(rank_prior,'n',{dim_num},1,rank_max,1);
end
%
switch data_type
    case 'syn'
        rank_min_default=0;
    case 'msi'
        rank_min_default=5;
    case 'hsi'
        rank_min_default=5;
    case 'real'
        rank_min_default=5;
end
rank_min=par_extract_mth(par,'rank_min',min(rank_min_default*ones(1,dim_num),rank_max-1));
input_check_mth(rank_min,'n',{dim_num},0,rank_max,1);
%
rank_idx_type=par_extract_mth(par,'rank_idx_type','gini');
input_check_mth(rank_idx_type,'c',{'lr_idx','gini'})
%
rank_thre=par_extract_mth(par,'rank_thre',1e-2);
input_check_mth(rank_thre,'n',{1},0,1)
%
rank_thre_min_ratio=par_extract_mth(par,'rank_thre_min_ratio',2/3);
input_check_mth(rank_thre_min_ratio,'n',{1},0,1)
rank_thre_min_ratio=1/rank_thre_min_ratio;
%
switch data_type
    case 'syn'
        xi0_inf_default=[20,100,500];
        cr_inf_default=[5e-2,2e-2,1e-2];
        drop_num_inf_default=[0.5,0.4,0.3;...
            0.5,0.4,0.3;...
            0.5,0.4,0.3];
    case 'msi'
        xi0_inf_default=[10,50,250];
        cr_inf_default=[5e-2,2e-2,1e-2];
        drop_num_inf_default=[0.2,0.15,0.1;...
            0.2,0.15,0.1;...
            0.3,0.2,0.15];
    case 'hsi'
        xi0_inf_default=[10,50,250];
        cr_inf_default=[5e-2,2e-2,1e-2];
        drop_num_inf_default=[0.4,0.35,0.3;...
            0.4,0.35,0.3;...
            0.1,0.075,0.05];
    case 'real'
        xi0_inf_default=[10,50,250];
        cr_inf_default=[5e-2,2e-2,1e-2];
        drop_num_inf_default=zeros(3);
end
rank_xi0_inf=par_extract_mth(par,'rank_xi0_inf',xi0_inf_default);
input_check_mth(rank_xi0_inf,'n',{dim_num},0)
rank_cr_inf=par_extract_mth(par,'rank_cr_inf',cr_inf_default);
input_check_mth(rank_cr_inf,'n',{dim_num},0)
rank_drop_num_inf=par_extract_mth(par,'rank_drop_num_inf',drop_num_inf_default);
input_check_mth(rank_drop_num_inf,'n',{[dim_num,3]},0,1)
%
switch data_type
    case 'syn'
        rank_sequ_length_default=20;
    case 'msi'
        rank_sequ_length_default=20;
    case {'hsi','real'}
        rank_sequ_length_default=5;
end
rank_sequ_length=par_extract_mth(par,'rank_sequ_length',rank_sequ_length_default);
input_check_mth(rank_sequ_length,'n',{1},1,[],1)
%
rank_sequ_power=par_extract_mth(par,'rank_sequ_power',5);
input_check_mth(rank_sequ_power,'n',{1},1,[],1)
%
par_rank=[];
par_rank.max=rank_max;
par_rank.init=rank_init;
par_rank.init2=rank_init2;
if exist('rank_prior','var');par_rank.prior=rank_prior;end
par_rank.min=rank_min;
par_rank.idx_type=rank_idx_type;
par_rank.thre=rank_thre;
par_rank.thre_min_ratio=rank_thre_min_ratio;
par_rank.xi0_inf=rank_xi0_inf;
par_rank.cr_inf=rank_cr_inf;
par_rank.drop_num_inf=rank_drop_num_inf;
par_rank.sequ_length=rank_sequ_length;
par_rank.sequ_power=rank_sequ_power;
%% par_mog
%
mog_num_default=8;
mog_num=par_extract_mth(par,'mog_num',mog_num_default);
input_check_mth(mog_num,'n',{1},1,[],1)
%
mog_num_thre_default=5e-2;
mog_num_thre=par_extract_mth(par,'mog_num_thre',mog_num_thre_default);
input_check_mth(mog_num_thre,'n',{1},0,1)
%
mog_num_tol=par_extract_mth(par,'mog_num_tol',1e-2);
input_check_mth(mog_num_tol,'n',{1},0,1)
%
par_mog=[];
par_mog.num=mog_num;
par_mog.thre=mog_num_thre;
par_mog.tol=mog_num_tol;
%% par_w
%
dim_active=par_extract_mth(par,'dim_active',1:dim_num,@(x)sort(reshape(x,1,[]),'ascend'));
input_check_mth(dim_active,'n',dim_num,1,dim_num,1,1)
%
w_eta=par_extract_mth(par,'w_eta',1);
input_check_mth(w_eta,'n',{1},0)
%
if isequal(stage_all,2)
    if isfield(par,'w_basic')
        w_basic=par.w_basic(:)';
    else
        error('no w_basic for stage_all=''2''')
    end
    input_check_mth(w_basic,'n',{dim_num},0);
    if sum(w_basic(dim_active))==0
        error('wrong w_basic and dim_active')
    end
    w_basic(dim_active)=w_basic(dim_active)/sum(w_basic(dim_active));
    w_basic(setdiff(1:dim_num,dim_active))=0;
end
%
dim_threshold=par_extract_mth(par,'dim_threshold',1e-2);
input_check_mth(dim_threshold,'n',{1},0,1)
%
switch data_type
    case 'syn'
        w_type_default='X';
    case 'msi'
        w_type_default='basic';
    case {'hsi','real'}
        w_type_default='basic';
end
w_type=par_extract_mth(par,'w_type',w_type_default);
input_check_mth(w_type,'c',{'X','basic','dim'});
%
par_w=[];
par_w.active=dim_active;
par_w.eta=w_eta;
if exist('w_basic','var');par_w.basic=w_basic;end
par_w.thre=dim_threshold;
par_w.type=w_type;
%% par_xi
%
xi_rate=par_extract_mth(par,'xi_rate',[1.5,1.5]);
input_check_mth(xi_rate,'n',{2},{1})
%
xi0_ini=par_extract_mth(par,'xi0_ini',[1,0.1]);
input_check_mth(xi0_ini,'n',{2},0)
%
xi_tol_X=par_extract_mth(par,'xi_tol_X',1e-2);
input_check_mth(xi_tol_X,'n',{1},0,1)
%
xi_tol_dim=par_extract_mth(par,'xi_tol_dim',1e-4);
input_check_mth(xi_tol_dim,'n',{1},0,1)
%
dim_min=1;
%
par_xi=[];
par_xi.rate=xi_rate;
par_xi.ini=xi0_ini;
par_xi.tol_X=xi_tol_X;
par_xi.tol_dim=xi_tol_dim;
par_xi.dim_min=dim_min;
%% par_model
par_model_default=1e-6;
%
a0=par_extract_mth(par,'a0',par_model_default);input_check_mth(a0,'n',{1},{0})
b0=par_extract_mth(par,'b0',par_model_default);input_check_mth(b0,'n',{1},{0})
%
e0=par_extract_mth(par,'e0',par_model_default);input_check_mth(e0,'n',{1},{0})
f0=par_extract_mth(par,'f0',par_model_default);input_check_mth(f0,'n',{1},{0})
%
alpha0=par_extract_mth(par,'alpha0',par_model_default);input_check_mth(alpha0,'n',{1},{0})
%
par_model=[];
par_model.a0=a0;
par_model.b0=b0;
par_model.e0=e0;
par_model.f0=f0;
par_model.alpha0=alpha0;
%%
par_all=[];
par_all.loc=par_loc;
par_all.dim=par_dim;
par_all.iter=par_iter;
par_all.rank=par_rank;
par_all.mog=par_mog;
par_all.w=par_w;
par_all.xi=par_xi;
par_all.model=par_model;
par_all.others=par_others;
end