function [p,q]=QAs_mth(Ref,X,qa,dim,m)
%%
if ~isequal(size(Ref),size(X))
    error('reference data and evaluated data have different sizes')
end
if ismatrix(Ref) && (~exist('dim','var') || isempty(dim))
    %
    dim=2;
end
input_check_mth(dim,'n',1,2,[],1);%
%%
maxRef=max(Ref(:));
maxX=max(X(:));
if abs(maxRef-maxX)>100
    warning('The intensity ranges of the two data may be different')
end
if ~exist('m','var') || isempty(m)
    if max(maxRef,maxX)>50
        m=255;
    else
        m=1;
    end
end
input_check_mth(m,'n',1,0);%
Ref=Ref*255/m;X=X*255/m;
%%
dimX=ndims(Ref);
sizeX=[size(Ref) ones(1,dim-dimX)];%
sizeUnit=sizeX(1:dim);%
q=zeros([sizeX(dim+1:end) 1]);
%
Ref=reshape(Ref,prod(sizeUnit),[]);
X=reshape(X,prod(sizeUnit),[]);
for i=1:numel(q)
    tempRef=reshape(Ref(:,i),sizeUnit);
    tempX=reshape(X(:,i),sizeUnit);
    switch qa
        case {'ReErr','reerr'}
            q(i)=norm(tempRef(:)-tempX(:),'fro')/norm(tempRef(:),'fro');
        case {'PSNR','psnr'}
            q(i)=10*log10(255^2./mean2((tempRef-tempX).^2));
        case {'SNR','snr'}
            %q(i)=10*log10(mean2((tempRef-mean2(tempRef)).^2)/mean2((tempRef-tempX).^2));
            q(i)=10*log10(mean2(tempRef.^2)/mean2((tempRef-tempX).^2));
        case {'SSIM','ssim'}
            q(i)=ssim(tempRef,tempX);
        case {'FSIM','fsim'}
            [tempq,tempqc]=FeatureSIM(tempRef,tempX);
            if dim==2
                q(i)=tempq;
            else
                q(i)=tempqc;
            end
        case {'ERGAS','ergas'}
            % [1] L. Wald, "Data Fusion: Definitions and ArchitecturesFusion of
            %     Images of Different Spatial Resolutions", Les Presses, Ecole des
            %     Mines de Paris, Paris, France (2002) 200 pp.
            if ~isequal(dim,3)
                warning('ERGAS is defined for 3-D data')
            end
            q(i)=100*sqrt(mean2(sum(sum((tempRef-tempX).^2,1),2)./sum(sum(tempRef,1),2)));
        case {'SAM','sam'}
            % [1] R. YUHAS, J. BOARDMAN, and A. GOETZ, "Determination of semi-arid
            %     landscape endmembers and seasonal trends using convex geometry
            %     spectral unmixing techniques", JPL, Summaries of the 4 th Annual JPL
            %     Airborne Geoscience Workshop. 1993.
            tmp=(sum(tempRef.*tempX,3)+eps)./(sqrt(sum(tempRef.^2,3))+eps)./(sqrt(sum(tempX.^2,3))+eps);
            q(i)=mean2(real(acos(tmp)));
        otherwise
            error('unknown quality assessment')
    end
end
q=double(q);p=mean2(q);
end