function input_check_mth(X,class_name,size_check,lower_bound,upper_bound,int_check,repetition_check)
%% 
flag=isempty(X);
err_msg='isempty';
subf_error(flag,err_msg,inputname(1));
%% 
if exist('class_name','var') && ~isempty(class_name)
    switch class_name
        case {'n','N','num','Num','numeric','Numeric'}%
            flag=~isnumeric(X);
            err_msg='isnumeric';
        case {'l','L','log','Log','logical','Logical'}%
            flag=~islogical(X);
            err_msg='islogical';
        case {'c','C','char','Char','s','S','str','Str','string','String'}%
            flag=~ischar(X);
            err_msg='ischar';
        otherwise
            error('unknown class')
    end
    subf_error(flag,err_msg,inputname(1));
end
%% 
if exist('size_check','var') && ~isempty(size_check)
    if ischar(size_check) || iscellstr(size_check)%
        flag=~any(strcmp(X,size_check));
        err_msg='char value';
    end
    if isnumeric(size_check)%
        if numel(size_check)==1%
            flag=numel(X)>size_check;
            err_msg='element number';
        else%
            flag=any(size(X)>size_check(:)');
            err_msg='size';
        end
    end
    if iscell(size_check) && isnumeric(size_check{1})%
        size_check=size_check{1};
        if numel(size_check)==1%
            flag=numel(X)~=size_check;
            err_msg='element number';
        else%
            flag=~isequal(size(X),size_check(:)');
            err_msg='size';
        end
    end
    if iscell(size_check) && ischar(size_check{1}) && strcmp(size_check{1},'dim') && (isnumeric(size_check{2}) || isnumeric(size_check{2}{1}))
        if isnumeric(size_check{2})%
            flag=ndims(X)>size_check{2};
        else
            flag=ndims(X)~=size_check{2};
        end
        err_msg='dimension number';
    end
    subf_error(flag,err_msg,inputname(1));
end
%% 
if exist('lower_bound','var') && ~isempty(lower_bound)
    if ~iscell(lower_bound)%
        flag=any(X(:)<lower_bound(:));
    else%
        flag=any(X(:)<=lower_bound{1}(:));
    end
    err_msg='lower bound';
    subf_error(flag,err_msg,inputname(1));
end
%% 
if exist('upper_bound','var') && ~isempty(upper_bound)
    if ~iscell(upper_bound)%
        flag=any(X(:)>upper_bound(:));
    else%
        flag=any(X(:)>=upper_bound{1}(:));
    end
    err_msg='upper bound';
    subf_error(flag,err_msg,inputname(1));
end
%% 
if exist('int_check','var') && ~isempty(int_check)
    flag=~isequal(X,fix(X));
    err_msg='integer';
    subf_error(flag,err_msg,inputname(1));
end 
%% 
if exist('repetition_check','var') && ~isempty(repetition_check)
    flag=~isequal(numel(X),numel(unique(X(:))));
    err_msg='repetitive elements';
    subf_error(flag,err_msg,inputname(1));
end 
end
%% 
function subf_error(flag,err_msg,name)
if flag
    error(['wrong when check ' err_msg ' for ' name]);
end
end